//
// Fan Contorl No.1
// AVR ATiny2313
// 2008.11.15
//

#include <avr/io.h>
#include <avr/eeprom.h>

#include "Utility.h"

#define LED_POWER	4		//Port B : LED of Power

#define LED_STATUS1	0		//Port D : LED of Status 1
#define LED_STATUS2	1		//Port D : LED of Status 2
#define	SW_STORE	2		//Port D : Swtich of Store
#define	DIP_F1		3		//Port D : DIP of Fan 1
#define DIP_F2		4		//Port D : DIP of Fan 2
#define	OUT_V1		5		//Port D : Sense volt 1
#define OUT_V2		6		//Port D : Sense volt 2

#define	SW_DOWN		0		//Port A : Swtich of Down
#define SW_UP		1		//Port A : Swtich of Up

#define SENSE_OK	10		//Check of Analog comparator
#define POWER_LED	5		//Power LED On/Off counter

#define FAN1_ADDRESS 0		//EEPROM address of Fan1 control data 
#define FAN2_ADDRESS 4		//EEPROM address of Fan2 control data

#define WAKE_UP		3000	//Wake up of FAN (300uSec * WAKE_UP)
#define	WAIT		300		//Wait of Loop   (300uSec * WAIT   )

static uint8_t fan1_cnt[4];
static uint8_t fan2_cnt[4];

void Fan_1( uint8_t unit )
{
	OCR0A = unit;
}

void Fan_2( uint8_t unit )
{
	OCR1AH = 0;
	OCR1AL = unit;
}

uint8_t Fan_ajst( uint8_t no, uint8_t ajst )
{
	uint8_t port_a0, port_a1, port_d0, port_d1, ajst_temp;

	if (no == 1) Fan_1( ajst );
	else         Fan_2( ajst );
	wait3( WAKE_UP );

	ajst_temp = ajst;
	port_a0 = PINA & (1<<SW_UP | 1<<SW_DOWN);
	port_a0 ^=       (1<<SW_UP | 1<<SW_DOWN);
	port_d0 = PIND & 1<<SW_STORE;
	port_d0 ^=       1<<SW_STORE;
	while (1) {
		port_a1 = PINA & (1<<SW_UP | 1<<SW_DOWN);
		port_a1 ^=       (1<<SW_UP | 1<<SW_DOWN);
		if ( port_a0 != port_a1 ) {						//Cansell
			if ((port_a1 & 1<<SW_UP) && (port_a1 & 1<<SW_DOWN)) break;
			if ( port_a1 & 1<<SW_UP)   ajst_temp += 8;	//Up
			if ( port_a1 & 1<<SW_DOWN) ajst_temp -= 8;	//Down
			if (no == 1) Fan_1( ajst_temp );
			else         Fan_2( ajst_temp );
			port_a0 = port_a1;
		}
		port_d1 = PIND & 1<<SW_STORE;
		port_d1 ^=       1<<SW_STORE;
		if ( port_d0 != port_d1 ) {
			if ( port_d1 & 1<<SW_STORE ) {
				ajst = ajst_temp;
				break;
			}
			port_d0 = port_d1;
		}
	}
	return ajst;
}

int main( void )
{
	unsigned char port_d0, port_d1;
	uint8_t  ajst, mode, sense, sense_level, sense_active;
	int pcnt, sense_on, sense_off;

	DDRA  = 0b00000000;
	DDRB  = 0b00011100;
	DDRD  = 0b01100011;
	PORTA = 0b00000011;
	PORTB = 0b00000000;
	PORTD = 0b00011100;

	ACSR  = 0b00000000;						// Analog comperator

	TCCR0A = 0b10000011;					// Timer8  Hi Speed PWM 1/1
	TCCR0B = 0b00000001;
	OCR0A  = 128;

	TCCR1A = 0b10000001;					// Timer16 Hi Speed PWM(8bit) 1/1
	TCCR1B = 0b00001001;
	OCR1AH = 0;
	OCR1AL = 128;

	eeprom_busy_wait();
	eeprom_read_block( fan1_cnt, (const void *)FAN1_ADDRESS, sizeof(fan1_cnt) );
	eeprom_busy_wait();
	eeprom_read_block( fan2_cnt, (const void *)FAN2_ADDRESS, sizeof(fan2_cnt) );

	port_d1 = PIND & (1<<DIP_F1 | 1<<DIP_F2);
	port_d1 ^=       (1<<DIP_F1 | 1<<DIP_F2);	// Initialaize of EEPROM DATA
	if ( (port_d1 & 1<<DIP_F1) && (port_d1 & 1<<DIP_F2) ) {
		fan1_cnt[0] = fan2_cnt[0] = 0;			// Fan Stop
		fan1_cnt[1] = fan2_cnt[1] = 160;		// Fan Low
		fan2_cnt[2] = fan2_cnt[2] = 200;		// Fan Mid
		fan2_cnt[3] = fan2_cnt[3] = 255;		// Fan Full
		eeprom_busy_wait();
		eeprom_write_block( fan1_cnt, (void *)FAN1_ADDRESS, sizeof(fan1_cnt) );
		eeprom_busy_wait();
		eeprom_write_block( fan2_cnt, (void *)FAN2_ADDRESS, sizeof(fan2_cnt) );
	}
	else if ( port_d1 & 1<<DIP_F1 ) {		// Adjustment of Mid Speed
		ajst = fan1_cnt[2];
		fan1_cnt[2] = Fan_ajst( 1, ajst );
		eeprom_busy_wait();
		eeprom_write_block( fan1_cnt, (void *)FAN1_ADDRESS, sizeof(fan1_cnt) );

		ajst = fan2_cnt[2];
		fan2_cnt[2] = Fan_ajst( 2, ajst );
		eeprom_busy_wait();
		eeprom_write_block( fan2_cnt, (void *)FAN2_ADDRESS, sizeof(fan2_cnt) );
	}
	else if ( port_d1 & 1<<DIP_F2 ) {	// Adjustment of Off Speed
		ajst = fan1_cnt[0];
		fan1_cnt[0] = Fan_ajst( 1, ajst );
		eeprom_busy_wait();
		eeprom_write_block( fan1_cnt, (void *)FAN1_ADDRESS, sizeof(fan1_cnt) );

		ajst = fan2_cnt[0];
		fan2_cnt[0] = Fan_ajst( 2, ajst );
		eeprom_busy_wait();
		eeprom_write_block( fan2_cnt, (void *)FAN2_ADDRESS, sizeof(fan2_cnt) );
	}
	while (1) {
		port_d1 = PIND & (1<<DIP_F1 | 1<<DIP_F2);
		port_d1 ^=       (1<<DIP_F1 | 1<<DIP_F2);
		if (port_d1 == 0) break;
	}

	pcnt  = 0;
	mode  = 0;
	sense = 4;
	sense_on  = 0;
	sense_off = 0;
	sense_level = 2;
	sense_active = 2;
	Fan_1( fan1_cnt[sense_active] );
	Fan_2( fan2_cnt[sense_active] );

	port_d0 = PIND & (1<<DIP_F1 | 1<<DIP_F2);
	port_d0 ^=       (1<<DIP_F1 | 1<<DIP_F2);
	while (1) {
		port_d1 = PIND & (1<<DIP_F1 | 1<<DIP_F2);
		port_d1 ^=       (1<<DIP_F1 | 1<<DIP_F2);
		if (port_d0 != port_d1) {
			if ( port_d1 & (1<<DIP_F1 | 1<<DIP_F2)) {		// Adjustment of fan Speed
				mode = 1;
				if ( port_d1 & 1<<DIP_F1 ) {
					Fan_1( fan1_cnt[2] );
					wait3( WAKE_UP );
					ajst = fan1_cnt[1];
					Fan_1( ajst );
					fan1_cnt[1] = Fan_ajst( 1, ajst );
					fan1_cnt[2] = 255 - (255 - fan1_cnt[1]) / 2;
					eeprom_busy_wait();
					eeprom_write_block( fan1_cnt, (void *)FAN1_ADDRESS, sizeof(fan1_cnt) );
				}
				if (port_d1 & 1<<DIP_F2) {
					Fan_2( fan2_cnt[2] );
					wait3( WAKE_UP );
					ajst = fan2_cnt[1];
					Fan_2( ajst );
					fan2_cnt[1] = Fan_ajst( 2, ajst );
					fan2_cnt[2] = 255 - (255 - fan2_cnt[1]) / 2;
					eeprom_busy_wait();
					eeprom_write_block( fan2_cnt, (void *)FAN2_ADDRESS, sizeof(fan2_cnt) );
				}
			}
			port_d0 = port_d1;
		}
		if (port_d1 == 0) {
			if (mode == 1) {
				Fan_1( fan1_cnt[sense_active]);
				Fan_2( fan2_cnt[sense_active]);
				mode = 0;
			}

			switch ( sense ) {
			case 0:										// Set to 22C
				PORTD &= ~_BV(OUT_V1);
				PORTD &= ~_BV(OUT_V2);
				sense_on  = 0;
				sense_off = 0;
				sense++;
				break;
			case 1:										// Sense
				if (ACSR & (1<<ACO)) {
					sense_on  = 0;
					sense_off++;
				}
				else {
					sense_on++;
					sense_off = 0;
				}
				if (sense_off >= SENSE_OK) {			// < 22C
					sense_off = SENSE_OK;
					sense_level = 0;					// fan stop
				}
				if (sense_on  >= SENSE_OK) {			// > 22C
					sense++;
					sense_level = 1;					// 22C - 35C
				}
				break;
			case 2:										// Set to 35C
				PORTD |=  _BV(OUT_V1);
				PORTD &= ~_BV(OUT_V2);
				sense_on  = 0;
				sense_off = 0;
				sense++;
				break;
			case 3:										// Sense
				if (ACSR & (1<<ACO)) {
					sense_on  = 0;
					sense_off++;
				}
				else {
					sense_on++;
					sense_off = 0;

				}
				if (sense_off >= SENSE_OK) {			// < 35C
					sense = 0;
				}
				if (sense_on  >= SENSE_OK) {			// > 35C
					sense++;
					sense_level = 2;
				}
				break;
			case 4:										// Set to 45C
				PORTD &= ~_BV(OUT_V1);
				PORTD |=  _BV(OUT_V2);
				sense_on  = 0;
				sense_off = 0;
				sense++;
				break;
			case 5:										// Sense
				if (ACSR & (1<<ACO)) {
					sense_on  = 0;
					sense_off++;
				}
				else {
					sense_on++;
					sense_off = 0;
				}
				if (sense_off >= SENSE_OK) {			// < 45C
					sense = 2;
				}
				if (sense_on  >= SENSE_OK) {			// > 45C
					sense = 5;
					sense_level = 3;
				}
				break;
			}
			if (sense_active != sense_level) {
				if (sense_active == 0) {
					Fan_1( fan1_cnt[2] );
					Fan_2( fan2_cnt[2] );
					wait3( WAKE_UP );
				}

				sense_active = sense_level;
				Fan_1( fan1_cnt[sense_active] );
				Fan_2( fan2_cnt[sense_active] );
				switch (sense_active) {
				case 0:
					PORTD &= ~_BV(LED_STATUS1);
					PORTD &= ~_BV(LED_STATUS2);
					break;
				case 1:
					PORTD |=  _BV(LED_STATUS1);
					PORTD &= ~_BV(LED_STATUS2);
					break;
				case 2:
					PORTD &= ~_BV(LED_STATUS1);
					PORTD |=  _BV(LED_STATUS2);
					break;
				case 3:
					PORTD |=  _BV(LED_STATUS1);
					PORTD |=  _BV(LED_STATUS2);
				}
			}
		}
		wait3( WAIT );
		if (pcnt++ > POWER_LED) {
			pcnt = 0;
			if (PORTB & 1<<LED_POWER) PORTB &= ~_BV(LED_POWER);
			else                      PORTB |=  _BV(LED_POWER);
		}
	}
}
